use anyhow::{
    Context,
    Result,
};
use clap::{
    Parser,
};
use feed_rs::{
    parser::parse,
    model::Feed,
};

use reqwest::{
    IntoUrl,
    Url,
};

#[derive(Parser)]
#[command(version, about, long_about = None)]
struct Cli {
    /// Path to file containing feeds
    #[arg(long, default_value_t = String::from("feeds.txt"))]
    feeds: String,
    /// Path to output file
    #[arg(long, default_value_t = String::from("index.html"))]
    out: String,
}

fn fetch_feed(url: impl IntoUrl) -> Result<Feed> {
    let content = reqwest::blocking::get(url)?.bytes()?;
    Ok(parse(&content[..])?)
}

fn read_feed_file(path: &str) -> Result<Vec<Url>> {
    let mut out = Vec::new();

    for line in std::fs::read_to_string(path)?.lines() {
        let line = line.trim().to_string();
        if line == "" {
            continue;
        }
        if line.starts_with("#") {
            continue;
        }
        let url = Url::parse(&line)?;
        println!("{}", url);

        out.push(url);
    }

    Ok(out)
}

fn main() -> Result<()> {
    let cli = Cli::parse();

    let mut feed_items = Vec::new();

    let feeds = read_feed_file(&cli.feeds)?;

    for feed_url in feeds {
        feed_items.extend(fetch_feed(feed_url)?.entries);
    }

    feed_items.sort_by(|a, b| a.published.unwrap_or(chrono::DateTime::UNIX_EPOCH).cmp(&b.published.unwrap_or(chrono::DateTime::UNIX_EPOCH)).reverse());

    let mut out = String::new();

    out.push_str("<!DOCTYPE html>\n");
    out.push_str("<html>\n");
    out.push_str("<head>\n");
    out.push_str("</head>\n");
    out.push_str("<body>\n");

    out.push_str("<ul>\n");

    for item in feed_items {
        out.push_str(&format!("<li><a href=\"{}\">{}</a> ({})</li>\n", item.links.first().context("No link found")?.href, item.title.context("No title found")?.content, item.published.context("No publishing time")?));
    }

    out.push_str("</ul>\n");

    out.push_str("</body>\n");
    out.push_str("</html>\n");

    std::fs::write(&cli.out, out)
        .context("Failed to write output to file")?;

    Ok(())
}
